package com.rookmotion.rook_sdk_health_connect

import android.app.Activity
import android.content.Context
import com.rookmotion.rook.sdk.RookConfigurationManager
import com.rookmotion.rook.sdk.RookEventManager
import com.rookmotion.rook.sdk.RookHealthPermissionsManager
import com.rookmotion.rook.sdk.RookSummaryManager
import com.rookmotion.rook.sdk.analytics.RookAnalytics
import com.rookmotion.rook.sdk.analytics.RookFramework
import com.rookmotion.rook.sdk.domain.environment.RookAndroidClass
import com.rookmotion.rook.tracker.StepsTracker
import com.rookmotion.rook_sdk_health_connect.extension.getByteArrayArgAt
import com.rookmotion.rook_sdk_health_connect.extension.getIntArgAt
import com.rookmotion.rook_sdk_health_connect.extension.getLongArgAt
import com.rookmotion.rook_sdk_health_connect.extension.getStringArgAt
import com.rookmotion.rook_sdk_health_connect.extension.intSuccess
import com.rookmotion.rook_sdk_health_connect.extension.resultBooleanError
import com.rookmotion.rook_sdk_health_connect.extension.resultBooleanSuccess
import com.rookmotion.rook_sdk_health_connect.extension.resultInt64Error
import com.rookmotion.rook_sdk_health_connect.extension.resultInt64Success
import com.rookmotion.rook_sdk_health_connect.extension.throwableError
import com.rookmotion.rook_sdk_health_connect.extension.toLocalDate
import com.rookmotion.rook_sdk_health_connect.mapper.toDomain
import com.rookmotion.rook_sdk_health_connect.mapper.toProto
import com.rookmotion.rook_sdk_health_connect.proto.ProtobufModels.HealthPermissionProto
import com.rookmotion.rook_sdk_health_connect.proto.ProtobufModels.RookConfigurationProto
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.SupervisorJob
import kotlinx.coroutines.launch
import java.time.Instant

/** RookSdkHealthConnectPlugin */
class RookSdkHealthConnectPlugin : FlutterPlugin, MethodCallHandler, ActivityAware {
    private lateinit var channel: MethodChannel
    private lateinit var scope: CoroutineScope

    private var context: Context? = null
    private var activity: Activity? = null

    private lateinit var rookConfigurationManager: RookConfigurationManager
    private lateinit var rookHealthPermissionsManager: RookHealthPermissionsManager
    private lateinit var rookSummaryManager: RookSummaryManager
    private lateinit var rookEventManager: RookEventManager

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        context = flutterPluginBinding.applicationContext

        rookConfigurationManager = RookConfigurationManager(flutterPluginBinding.applicationContext)
        rookHealthPermissionsManager = RookHealthPermissionsManager(rookConfigurationManager)
        rookSummaryManager = RookSummaryManager(rookConfigurationManager)
        rookEventManager = RookEventManager(rookConfigurationManager)

        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "rook_sdk_health_connect")
        scope = CoroutineScope(SupervisorJob() + Dispatchers.IO)
        channel.setMethodCallHandler(this)
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "enableNativeLogs" -> {
                rookConfigurationManager.enableLocalLogs()

                result.resultBooleanSuccess(true)
            }

            "setConfiguration" -> {
                val bytes = call.getByteArrayArgAt(0)
                val proto = RookConfigurationProto.parseFrom(bytes)

                rookConfigurationManager.setConfiguration(proto.toDomain())

                result.resultBooleanSuccess(true)
            }

            "getUserID" -> {
                val userID = rookConfigurationManager.getUserID()

                result.success(userID)
            }

            "initRook" -> scope.launch {
                RookAnalytics.setFramework(RookFramework.FLUTTER)

                rookConfigurationManager.initRook().fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "updateUserID" -> scope.launch {
                val userID = call.getStringArgAt(0)

                rookConfigurationManager.updateUserID(userID).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "clearUserID" -> {
                rookConfigurationManager.clearUserID()

                result.resultBooleanSuccess(true)
            }

            "deleteUserFromRook" -> scope.launch {
                rookConfigurationManager.deleteUserFromRook().fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncUserTimeZone" -> scope.launch {
                rookConfigurationManager.syncUserTimeZone().fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "checkAvailability" -> {
                try {
                    val hcAvailabilityStatus = RookHealthPermissionsManager.checkAvailability(
                        context!!,
                    )
                    val proto = hcAvailabilityStatus.toProto()

                    result.intSuccess(proto.number)
                } catch (e: NullPointerException) {
                    result.throwableError(e)
                }
            }

            "openHealthConnectSettings" -> scope.launch {
                rookHealthPermissionsManager.openHealthConnectSettings().fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "checkPermissions" -> scope.launch {
                val arg1 = call.getIntArgAt(0).let {
                    HealthPermissionProto.forNumber(it).toDomain()
                }

                rookHealthPermissionsManager.checkPermissions(arg1).fold(
                    {
                        result.resultBooleanSuccess(it)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "requestPermissions" -> {
                val arg1 = call.getIntArgAt(0).let {
                    HealthPermissionProto.forNumber(it).toDomain()
                }

                try {
                    HealthConnectPermissionsActivity.launch(context!!, arg1)

                    result.resultBooleanSuccess(true)
                } catch (e: NullPointerException) {
                    result.resultBooleanError(e)
                }
            }

            "shouldSyncSleepSummariesFor" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.shouldSyncSleepSummariesFor(date).fold(
                    {
                        result.resultBooleanSuccess(it)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncSleepSummary" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.syncSleepSummary(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "shouldSyncBodySummariesFor" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.shouldSyncBodySummariesFor(date).fold(
                    {
                        result.resultBooleanSuccess(it)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncBodySummary" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.syncBodySummary(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "shouldSyncPhysicalSummariesFor" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.shouldSyncPhysicalSummariesFor(date).fold(
                    {
                        result.resultBooleanSuccess(it)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncPhysicalSummary" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookSummaryManager.syncPhysicalSummary(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncPendingSummaries" -> scope.launch {
                rookSummaryManager.syncPendingSummaries().fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncYesterdaySummaries" -> scope.launch {
                rookSummaryManager.syncYesterdaySummaries()
                result.resultBooleanSuccess(true)
            }

            "syncPhysicalEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncPhysicalEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncBloodGlucoseEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBloodGlucoseEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncBloodPressureEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBloodPressureEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncBodyMetricsEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBodyMetricsEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncBodyHeartRateEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBodyHeartRateEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncPhysicalHeartRateEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncPhysicalHeartRateEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncHydrationEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncHydrationEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncNutritionEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncNutritionEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncBodyOxygenationEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncBodyOxygenationEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncPhysicalOxygenationEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncPhysicalOxygenationEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncTemperatureEvents" -> scope.launch {
                val millis = call.getLongArgAt(0)
                val date = Instant.ofEpochMilli(millis).toLocalDate()

                rookEventManager.syncTemperatureEvents(date).fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncPendingEvents" -> scope.launch {
                rookEventManager.syncPendingEvents().fold(
                    {
                        result.resultBooleanSuccess(true)
                    },
                    {
                        result.resultBooleanError(it)
                    }
                )
            }

            "syncYesterdayEvents" -> scope.launch {
                rookEventManager.syncYesterdayEvents()
                result.resultBooleanSuccess(true)
            }

            "isStepsTrackerAvailable" -> {
                try {
                    val isAvailable = StepsTracker.isAvailable(context!!)

                    result.resultBooleanSuccess(isAvailable)
                } catch (e: NullPointerException) {
                    result.resultBooleanError(e)
                }
            }

            "isStepsTrackerActive" -> scope.launch {
                val isActive = StepsTracker.isActive()

                result.resultBooleanSuccess(isActive)
            }

            "hasStepsTrackerPermissions" -> {
                try {
                    val hasPermissions = StepsTracker.hasPermissions(context!!)

                    result.resultBooleanSuccess(hasPermissions)
                } catch (e: NullPointerException) {
                    result.resultBooleanError(e)
                }
            }

            "requestStepsTrackerPermissions" -> {
                try {
                    StepsTracker.requestPermissions(context!!)

                    result.resultBooleanSuccess(true)
                } catch (e: NullPointerException) {
                    result.resultBooleanError(e)
                }
            }

            "startStepsTracker" -> {
                try {
                    StepsTracker.start(context!!).fold(
                        {
                            result.resultBooleanSuccess(true)
                        },
                        {
                            result.resultBooleanError(it)
                        }
                    )
                } catch (e: NullPointerException) {
                    result.resultBooleanError(e)
                }
            }

            "stopStepsTracker" -> {
                try {
                    StepsTracker.stop(context!!).fold(
                        {
                            result.resultBooleanSuccess(true)
                        },
                        {
                            result.resultBooleanError(it)
                        }
                    )
                } catch (e: NullPointerException) {
                    result.resultBooleanError(e)
                }
            }

            "getTodaySteps" -> scope.launch {
                StepsTracker.getTodaySteps().fold(
                    {
                        result.resultInt64Success(it)
                    },
                    {
                        result.resultInt64Error(it)
                    }
                )
            }

            else -> result.notImplemented()
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        initializeActivity(binding)
    }

    override fun onDetachedFromActivityForConfigChanges() {
        destroyActivity()
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
        initializeActivity(binding)
    }

    override fun onDetachedFromActivity() {
        destroyActivity()
    }

    private fun initializeActivity(binding: ActivityPluginBinding) {
        try {
            activity = binding.activity
        } catch (ignored: Exception) {
            // Ignored
        }
    }

    private fun destroyActivity() {
        try {
            activity = null
        } catch (ignored: Exception) {
            // Ignored
        }
    }

    private fun test() {
        if (BuildConfig.DEBUG) {
            RookAndroidClass.test()
        } else {
            RookAndroidClass.disableTest()
        }
    }
}
